==> phoenix-0.1.7.tar.gz: phoenix-0.1.7/scripts/anchor.tcl <==
################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

# anchor.tcl
#
# Global variables that thrive here:
#
# phxW3AnchorBox	The name of the dialog box window
# tkW3HtAnchors		Array with NAME and HREF data.
# phxW3AnchorText	The start and stop places of the anchor within the text
# phxW3AnchorTagName	Index into tkW3HtAnchors array

# Make sure that form widgets can't become anchors except where appropriate!!!

proc phxW3AnchorInitialize {} {
    global phxW3AnchorBox
    set phxW3AnchorBox .editanchor
}

proc phxW3AnchorDialog {} {

    global phxW3AnchorText
    set w [phxW3SelectionGet]
    set left [phxW3SelectionGetLeftOfFocus]
    set right [phxW3SelectionGetLeftOfFocus]

    if {[phxW3ModeGet] != "edit" || $w == ""} return
    switch [phxW3WidgetClass $w] {
	para {set phxW3AnchorText [$w tag ranges sel]}
	image {set phxW3AnchorText {{} {}}}
	default {return}
    }

    phxW3AnchorShowDialog $w $left $right
    phxW3AnchorShowData $w
}

proc phxW3AnchorShowDialog {w left right} {

    global phxW3AnchorBox
    global docsource doctype

    if [winfo exists $phxW3AnchorBox] {
	wm deiconify $phxW3AnchorBox
	wm transient $phxW3AnchorBox $w
	$phxW3AnchorBox.buttonframe.ok configure \
	    -command "phxW3AnchorSave $w $left $right
wm withdraw $phxW3AnchorBox"
	return
    }

    toplevel $phxW3AnchorBox -height 225 -width 400
    wm transient $phxW3AnchorBox $w
    wm title $phxW3AnchorBox "Edit Anchor"
    wm geometry $phxW3AnchorBox 500x200+70+180
    wm maxsize $phxW3AnchorBox 640 480

    frame $phxW3AnchorBox.destframe -relief flat
    frame $phxW3AnchorBox.nameframe -relief flat
    frame $phxW3AnchorBox.buttonframe -relief flat

    place  $phxW3AnchorBox.nameframe -in $phxW3AnchorBox \
	-rely 0 -relheight .85 -relwidth 1
    place $phxW3AnchorBox.buttonframe -in $phxW3AnchorBox \
	-rely .85 -relheight .15 -relwidth 1

    label $phxW3AnchorBox.destframe.label -text "Destination Document"
    radiobutton $phxW3AnchorBox.destframe.local \
	-text "Local File" \
	-anchor w \
	-relief flat \
	-command "phxW3EditFillFiles $phxW3AnchorBox.destframe.files " \
	-variable docsource
    radiobutton $phxW3AnchorBox.destframe.remote \
	-text "Remote File" \
	-anchor w \
	-relief flat \
	-variable docsource
    radiobutton $phxW3AnchorBox.destframe.gopher \
	-text "Gopher" \
	-anchor w \
	-relief flat \
	-variable docsource

    place $phxW3AnchorBox.destframe.label -in $phxW3AnchorBox.destframe
    place $phxW3AnchorBox.destframe.local -in $phxW3AnchorBox.destframe \
	-rely .20 
    place $phxW3AnchorBox.destframe.remote -in $phxW3AnchorBox.destframe \
	-rely .40
    place $phxW3AnchorBox.destframe.gopher -in $phxW3AnchorBox.destframe \
	-rely .60

    radiobutton $phxW3AnchorBox.destframe.html -text "HTML" -anchor w \
	-relief flat -variable doctype
    radiobutton $phxW3AnchorBox.destframe.text -text "Text" -anchor w \
	-relief flat -variable doctype
    radiobutton $phxW3AnchorBox.destframe.image -text Image -anchor w \
	-relief flat -variable doctype

    place $phxW3AnchorBox.destframe.html -in $phxW3AnchorBox.destframe \
	-relx .25 -rely .2 
    place $phxW3AnchorBox.destframe.text -in $phxW3AnchorBox.destframe \
	-relx .25 -rely .4 
    place $phxW3AnchorBox.destframe.image -in $phxW3AnchorBox.destframe \
	-relx .25 -rely .6 

    listbox $phxW3AnchorBox.destframe.files -relief sunken

    bind $phxW3AnchorBox.destframe.files <Button-1> {
	global phxW3curdir
	%W select from [%W nearest %y]
	set file [lindex [selection get] 0]
	if {$file != ""} {
	    if {[file isdirectory $file] == 1} {
		phxW3EditFillFiles \
		    %W [file dirname $phxW3curdir]/$file
	    }
	}
    }
	
    place $phxW3AnchorBox.destframe.files -in $phxW3AnchorBox.destframe \
	-relx .50 -rely .2 -relwidth .45 -relheight .6

    $phxW3AnchorBox.destframe.local invoke
    $phxW3AnchorBox.destframe.html invoke

    #label $phxW3AnchorBox.nameframe.textlabel -text "Anchor text"
    #place $phxW3AnchorBox.nameframe.textlabel -in $phxW3AnchorBox.nameframe \
	-rely 0

    label $phxW3AnchorBox.nameframe.text -fg red \
	-font -adobe-times-bold-r-normal--24-240-75-75-p-132-iso8859-1
    place $phxW3AnchorBox.nameframe.text -in $phxW3AnchorBox.nameframe \
	-rely 0 -relwidth .95

    label $phxW3AnchorBox.nameframe.addrlabel -text "Address"
    place $phxW3AnchorBox.nameframe.addrlabel -in $phxW3AnchorBox.nameframe \
	-rely .2

    entry $phxW3AnchorBox.nameframe.addr -relief sunken
    phxW3SelectionWidgetInitialize $phxW3AnchorBox.nameframe.addr
    bind $phxW3AnchorBox.nameframe.addr <Return> \
	"phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.name"
    bind $phxW3AnchorBox.nameframe.addr <Tab> \
	"phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.name"
    place $phxW3AnchorBox.nameframe.addr -in $phxW3AnchorBox.nameframe \
	-rely .4 -relx .025 -relwidth .95

    label $phxW3AnchorBox.nameframe.namelabel -text "Name of Link (optional)"
    place $phxW3AnchorBox.nameframe.namelabel -in $phxW3AnchorBox.nameframe \
	-rely .6

    entry $phxW3AnchorBox.nameframe.name -relief sunken
    phxW3SelectionWidgetInitialize $phxW3AnchorBox.nameframe.name
    bind $phxW3AnchorBox.nameframe.name <Return> \
	"phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.addr"
    bind $phxW3AnchorBox.nameframe.name <Tab> \
	"phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.addr"
    place $phxW3AnchorBox.nameframe.name -in $phxW3AnchorBox.nameframe \
	-rely .80 -relx .025 -relwidth .95



    button $phxW3AnchorBox.buttonframe.ok -text OK \
	-command "phxW3AnchorSave $w $left $right
wm withdraw $phxW3AnchorBox"
    place $phxW3AnchorBox.buttonframe.ok -in $phxW3AnchorBox.buttonframe \
	-relx .2 

    button $phxW3AnchorBox.buttonframe.cancel -text Cancel \
	-command "wm withdraw $phxW3AnchorBox
phxW3SelectionSetFocus $left $right"
    place $phxW3AnchorBox.buttonframe.cancel -in $phxW3AnchorBox.buttonframe \
	-relx .75
}

proc phxW3AnchorShowData {w} {

    global phxW3AnchorBox
    global tkW3HtAnchors tkW3HtImages phxW3AnchorTagName phxW3AnchorText

    set class [phxW3WidgetClass $w]
    if {$class == "para"} {
	# If the selected range is completely within an existing anchor then
	# expand to the whole anchor.  Is this what we want?!!
	if {[regexp {h([0-9]+)} \
		[$w tag names [lindex $phxW3AnchorText 0]] "" aiFirst] && \
	    [regexp {h([0-9]+)} \
		[$w tag names [lindex $phxW3AnchorText 1]-1c] "" aiLast] && \
	    $aiFirst == $aiLast} \
	{
	    set ai $aiFirst
	    $w tag add sel h$ai.first h$ai.last
	    set phxW3AnchorText [$w tag ranges h$ai]
	}
    } elseif {$tkW3HtImages($tkW3HtImages($w).anchortag) != ""} {
	set ai $tkW3HtImages($tkW3HtImages($w).anchortag)
    }

    $phxW3AnchorBox.nameframe.addr delete 0 end
    $phxW3AnchorBox.nameframe.name delete 0 end

    if [info exists ai] {
	$phxW3AnchorBox.nameframe.addr insert 0 $tkW3HtAnchors(href.$ai)
	$phxW3AnchorBox.nameframe.name insert 0 $tkW3HtAnchors(name.$ai)
	set phxW3AnchorTagName $ai
    } else {
	global tkW3ConfigHomePage
	$phxW3AnchorBox.nameframe.addr insert 0 $tkW3ConfigHomePage
	set phxW3AnchorTagName ""
    }

    if {$class == "para"} {
	$phxW3AnchorBox.nameframe.text config -text [$w get \
	    [lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]]
    } else {
	$phxW3AnchorBox.nameframe.text config -text "Image"
    }

    phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.addr
    $phxW3AnchorBox.nameframe.addr icursor end
}

proc phxW3AnchorSave {w left right} {

    global phxW3AnchorBox
    global phxW3AnchorText phxW3AnchorTagName
    global tkW3HtAnchors tkW3HtImages tkW3HtMaxAnchor

    set class [phxW3WidgetClass $w]

    # Restore the focus and selection.
    phxW3SelectionSet $w
    phxW3SelectionSetFocus $left $right

    # If $w is a para widget, remove existing anchors and set the sel tag.
    # We do it here because we have to do it whether we are creating,
    # changing, or destroying an anchor.
    if {$class == "para"} {
	$w tag add sel [lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]

	# Delete any existing anchors in this range
	$w tag remove Anchor \
	    [lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]
	for {set i 0} {$i <= $tkW3HtMaxAnchor} {incr i} {
	    $w tag remove h$i \
		[lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]
	}
    }

    if {[$phxW3AnchorBox.nameframe.addr get] != "" ||
	[$phxW3AnchorBox.nameframe.name get] != ""} \
    {
	# Change or create an anchor.
	# First, configure the anchor correctly.
	if {$phxW3AnchorTagName == ""} {
	    # We're creating a new anchor.
	    set phxW3AnchorTagName $tkW3HtAnchors(index)
	    incr tkW3HtMaxAnchor
	    incr tkW3HtAnchors(index)

	    set tkW3HtAnchors(href.$phxW3AnchorTagName) \
		[string trim [$phxW3AnchorBox.nameframe.addr get]]
	    set tkW3HtAnchors(name.$phxW3AnchorTagName) \
		[string trim [$phxW3AnchorBox.nameframe.name get]]

	    if {$class == "para"} {
		tkW3HtMakeAnchor $phxW3AnchorTagName \
		    $tkW3HtAnchors(name.$phxW3AnchorTagName) \
		    $tkW3HtAnchors(href.$phxW3AnchorTagName)
	    } elseif {$class == "image"} {
		phxW3AnchorToggleImage $w 1 $phxW3AnchorTagName 0
		set class [phxW3WidgetClass $w]
	    }
	} else {
	    # We're changing the anchor
	    set tkW3HtAnchors(href.$phxW3AnchorTagName) \
		[string trim [$phxW3AnchorBox.nameframe.addr get]]
	    set tkW3HtAnchors(name.$phxW3AnchorTagName) \
		[string trim [$phxW3AnchorBox.nameframe.name get]]
	    tkW3HtMakeAnchor $phxW3AnchorTagName \
		$tkW3HtAnchors(name.$phxW3AnchorTagName) \
		$tkW3HtAnchors(href.$phxW3AnchorTagName)
	}

	# Second, configure the widget to refer to the anchor.
	if {$class == "para"} {
	    $w tag add Anchor \
		[lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]
	    $w tag add h$phxW3AnchorTagName \
		[lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]
	} elseif {$class == "image"} {
	    $w configure -command \
		"phxW3ModeSingleClickAnchor %W $phxW3AnchorTagName"
	    bind $w <Double-Button-1> \
		"phxW3ModeDoubleClickAnchor %W $phxW3AnchorTagName"
	}
    } else {
	# We're deleting an anchor.
	set tkW3HtAnchors(href.$phxW3AnchorTagName) ""
	set tkW3HtAnchors(name.$phxW3AnchorTagName) ""

	# For Text widgets: We deleted all tags above.
	# For Buttons: we must turn them into Labels.
	if {$class == "image"} {
	    phxW3AnchorToggleImage $w 0 0 0
	    set class [phxW3WidgetClass $w]
	}
    }
    phxW3SaveDirtyBit
}

# phxW3AnchorToggleImage
#
# w = Label/Button to be made into a Button/Label
# anchor = whether its being converted to an anchor
# index = index of anchor (if it is one)
# ismap = whether it is an ismap

proc phxW3AnchorToggleImage {w anchor index ismap} {
    global tkW3HtImages tkW3HtAnchors

    # Put in a dummy frame as a place holder.
    pack [frame .atiTemp] -after $w -side left

    # Extract the bitmap.  Get the index for information in tkW3HtImages.
    set bitmap \
	[string range [lindex [$w configure -bitmap] 4] 1 end]
    set i $tkW3HtImages($w)

    # Wipe out the old label/button.  Re-install the tkW3HtImages information.
    # We unset and reset tkW3HtImages($w) so that the destroy command (via
    # the binding) does not destroy the information in tkW3HtImages.
    unset tkW3HtImages($w)
    destroy $w
    set tkW3HtImages($w) $i

    # Create a new button/label according to specification
    phxW3WidgetCreateImage $w $bitmap $anchor $index $ismap
    pack $w -before .atiTemp -side left -anchor w
    if {$anchor} {
	set tkW3HtImages($i.anchor) $tkW3HtAnchors(href.$index)
	set tkW3HtImages($i.anchortag) $index
    } else {
	set tkW3HtImages($i.anchor) ""
	set tkW3HtImages($i.anchortag) ""
    }


    # Destroy the place holder
    destroy .atiTemp
}

proc phxW3AnchorCurrentHref {} {

    global phxW3AnchorBox

    return [string trim [$phxW3AnchorBox.nameframe.addr get]]
}

proc phxW3AnchorBroadcastHref {h} {
	#set h [phxW3InfoConvertToHTTP $h]
	phxW3AnchorChangeHref $h
	phxW3NavigateSendToTkWWW [list phxW3AnchorChangeHref $h]
}		

proc phxW3AnchorChangeHref {h} {

    global phxW3AnchorBox

    if {[info exists phxW3AnchorBox] && \
	[winfo exists $phxW3AnchorBox.nameframe.addr]} \
    {
	$phxW3AnchorBox.nameframe.addr delete 0 end
	$phxW3AnchorBox.nameframe.addr insert 0 $h
    }
}

==> phoenix-0.1.8.tar.gz: phoenix-0.1.8/scripts/anchor.tcl <==
################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

# anchor.tcl
#
# Global variables that thrive here:
#
# phxW3AnchorBox	The name of the dialog box window
# tkW3HtAnchors		Array with NAME and HREF data.
# phxW3AnchorText	The start and stop places of the anchor within the text
# phxW3AnchorTagName	Index into tkW3HtAnchors array

# Make sure that form widgets can't become anchors except where appropriate!!!

proc phxW3AnchorInitialize {} {
    global phxW3AnchorBox
    set phxW3AnchorBox .editanchor
}

proc phxW3AnchorDialog {} {

    global phxW3AnchorText
    set w [phxW3SelectionGet]

    if {[phxW3ModeGet] != "edit" || $w == ""} return
    switch [phxW3WidgetClass $w] {
	para {set phxW3AnchorText [$w tag ranges sel]}
	image {set phxW3AnchorText {{} {}}}
	default {return}
    }

    # Record the current selection
    phxW3SelectionPush

    phxW3AnchorShowDialog $w
    phxW3AnchorShowData $w
}

proc phxW3AnchorShowDialog {w} {

    global phxW3AnchorBox
    global docsource doctype

    if [winfo exists $phxW3AnchorBox] {
	wm deiconify $phxW3AnchorBox
	wm transient $phxW3AnchorBox $w
	$phxW3AnchorBox.buttonframe.ok configure \
	    -command "phxW3AnchorSave $w\nwm withdraw $phxW3AnchorBox"
	return
    }

    toplevel $phxW3AnchorBox -height 225 -width 400
    wm transient $phxW3AnchorBox $w
    wm title $phxW3AnchorBox "Edit Anchor"
    wm geometry $phxW3AnchorBox 500x200+70+180
    wm maxsize $phxW3AnchorBox 640 480

    frame $phxW3AnchorBox.destframe -relief flat
    frame $phxW3AnchorBox.nameframe -relief flat
    frame $phxW3AnchorBox.buttonframe -relief flat

    place  $phxW3AnchorBox.nameframe -in $phxW3AnchorBox \
	-rely 0 -relheight .85 -relwidth 1
    place $phxW3AnchorBox.buttonframe -in $phxW3AnchorBox \
	-rely .85 -relheight .15 -relwidth 1

    label $phxW3AnchorBox.destframe.label -text "Destination Document"
    radiobutton $phxW3AnchorBox.destframe.local \
	-text "Local File" \
	-anchor w \
	-relief flat \
	-command "phxW3EditFillFiles $phxW3AnchorBox.destframe.files " \
	-variable docsource
    radiobutton $phxW3AnchorBox.destframe.remote \
	-text "Remote File" \
	-anchor w \
	-relief flat \
	-variable docsource
    radiobutton $phxW3AnchorBox.destframe.gopher \
	-text "Gopher" \
	-anchor w \
	-relief flat \
	-variable docsource

    place $phxW3AnchorBox.destframe.label -in $phxW3AnchorBox.destframe
    place $phxW3AnchorBox.destframe.local -in $phxW3AnchorBox.destframe \
	-rely .20 
    place $phxW3AnchorBox.destframe.remote -in $phxW3AnchorBox.destframe \
	-rely .40
    place $phxW3AnchorBox.destframe.gopher -in $phxW3AnchorBox.destframe \
	-rely .60

    radiobutton $phxW3AnchorBox.destframe.html -text "HTML" -anchor w \
	-relief flat -variable doctype
    radiobutton $phxW3AnchorBox.destframe.text -text "Text" -anchor w \
	-relief flat -variable doctype
    radiobutton $phxW3AnchorBox.destframe.image -text Image -anchor w \
	-relief flat -variable doctype

    place $phxW3AnchorBox.destframe.html -in $phxW3AnchorBox.destframe \
	-relx .25 -rely .2 
    place $phxW3AnchorBox.destframe.text -in $phxW3AnchorBox.destframe \
	-relx .25 -rely .4 
    place $phxW3AnchorBox.destframe.image -in $phxW3AnchorBox.destframe \
	-relx .25 -rely .6 

    listbox $phxW3AnchorBox.destframe.files -relief sunken

    bind $phxW3AnchorBox.destframe.files <Button-1> {
	global phxW3curdir
	%W select from [%W nearest %y]
	set file [lindex [selection get] 0]
	if {$file != ""} {
	    if {[file isdirectory $file] == 1} {
		phxW3EditFillFiles \
		    %W [file dirname $phxW3curdir]/$file
	    }
	}
    }
	
    place $phxW3AnchorBox.destframe.files -in $phxW3AnchorBox.destframe \
	-relx .50 -rely .2 -relwidth .45 -relheight .6

    $phxW3AnchorBox.destframe.local invoke
    $phxW3AnchorBox.destframe.html invoke

    #label $phxW3AnchorBox.nameframe.textlabel -text "Anchor text"
    #place $phxW3AnchorBox.nameframe.textlabel -in $phxW3AnchorBox.nameframe \
	-rely 0

    label $phxW3AnchorBox.nameframe.text -fg red \
	-font -adobe-times-bold-r-normal--24-240-75-75-p-132-iso8859-1
    place $phxW3AnchorBox.nameframe.text -in $phxW3AnchorBox.nameframe \
	-rely 0 -relwidth .95

    label $phxW3AnchorBox.nameframe.addrlabel -text "Address"
    place $phxW3AnchorBox.nameframe.addrlabel -in $phxW3AnchorBox.nameframe \
	-rely .2

    entry $phxW3AnchorBox.nameframe.addr -relief sunken
    phxW3SelectionWidgetInitialize $phxW3AnchorBox.nameframe.addr
    bind $phxW3AnchorBox.nameframe.addr <Return> \
	"phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.name"
    bind $phxW3AnchorBox.nameframe.addr <Tab> \
	"phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.name"
    place $phxW3AnchorBox.nameframe.addr -in $phxW3AnchorBox.nameframe \
	-rely .4 -relx .025 -relwidth .95

    label $phxW3AnchorBox.nameframe.namelabel -text "Name of Link (optional)"
    place $phxW3AnchorBox.nameframe.namelabel -in $phxW3AnchorBox.nameframe \
	-rely .6

    entry $phxW3AnchorBox.nameframe.name -relief sunken
    phxW3SelectionWidgetInitialize $phxW3AnchorBox.nameframe.name
    bind $phxW3AnchorBox.nameframe.name <Return> \
	"phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.addr"
    bind $phxW3AnchorBox.nameframe.name <Tab> \
	"phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.addr"
    place $phxW3AnchorBox.nameframe.name -in $phxW3AnchorBox.nameframe \
	-rely .80 -relx .025 -relwidth .95



    button $phxW3AnchorBox.buttonframe.ok -text OK \
	-command "phxW3AnchorSave $w\nwm withdraw $phxW3AnchorBox"
    place $phxW3AnchorBox.buttonframe.ok -in $phxW3AnchorBox.buttonframe \
	-relx .2 

    button $phxW3AnchorBox.buttonframe.cancel -text Cancel \
	-command "wm withdraw $phxW3AnchorBox\nphxW3SelectionPop"
    place $phxW3AnchorBox.buttonframe.cancel -in $phxW3AnchorBox.buttonframe \
	-relx .75
}

proc phxW3AnchorShowData {w} {

    global phxW3AnchorBox
    global tkW3HtAnchors tkW3HtImages phxW3AnchorTagName phxW3AnchorText

    set class [phxW3WidgetClass $w]
    if {$class == "para"} {
	# If the selected range is completely within an existing anchor then
	# expand to the whole anchor.  Is this what we want?!!
	if {[regexp {h([0-9]+)} \
		[$w tag names [lindex $phxW3AnchorText 0]] "" aiFirst] && \
	    [regexp {h([0-9]+)} \
		[$w tag names [lindex $phxW3AnchorText 1]-1c] "" aiLast] && \
	    $aiFirst == $aiLast} \
	{
	    set ai $aiFirst
	    $w tag add sel h$ai.first h$ai.last
	    set phxW3AnchorText [$w tag ranges h$ai]
	}
    } elseif {$tkW3HtImages($tkW3HtImages($w).anchortag) != ""} {
	set ai $tkW3HtImages($tkW3HtImages($w).anchortag)
    }

    $phxW3AnchorBox.nameframe.addr delete 0 end
    $phxW3AnchorBox.nameframe.name delete 0 end

    if [info exists ai] {
	$phxW3AnchorBox.nameframe.addr insert 0 $tkW3HtAnchors(href.$ai)
	$phxW3AnchorBox.nameframe.name insert 0 $tkW3HtAnchors(name.$ai)
	set phxW3AnchorTagName $ai
    } else {
	global tkW3ConfigHomePage
	$phxW3AnchorBox.nameframe.addr insert 0 $tkW3ConfigHomePage
	set phxW3AnchorTagName ""
    }

    if {$class == "para"} {
	$phxW3AnchorBox.nameframe.text config -text [$w get \
	    [lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]]
    } else {
	$phxW3AnchorBox.nameframe.text config -text "Image"
    }

    phxW3SelectionSetFocus $phxW3AnchorBox.nameframe.addr
    $phxW3AnchorBox.nameframe.addr icursor end
}

proc phxW3AnchorSave {w} {

    global phxW3AnchorBox
    global phxW3AnchorText phxW3AnchorTagName
    global tkW3HtAnchors tkW3HtImages tkW3HtMaxAnchor

    set class [phxW3WidgetClass $w]

    # Restore the focus and selection.
    phxW3SelectionPop

    # If $w is a para widget, remove existing anchors and set the sel tag.
    # We do it here because we have to do it whether we are creating,
    # changing, or destroying an anchor.
    if {$class == "para"} {
	# Delete any existing anchors in this range
	$w tag remove Anchor \
	    [lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]
	for {set i 0} {$i <= $tkW3HtMaxAnchor} {incr i} {
	    $w tag remove h$i \
		[lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]
	}
    }

    if {[$phxW3AnchorBox.nameframe.addr get] != "" ||
	[$phxW3AnchorBox.nameframe.name get] != ""} \
    {
	# Change or create an anchor.
	# First, configure the anchor correctly.
	if {$phxW3AnchorTagName == ""} {
	    # We're creating a new anchor.
	    set phxW3AnchorTagName $tkW3HtAnchors(index)
	    incr tkW3HtMaxAnchor
	    incr tkW3HtAnchors(index)

	    set tkW3HtAnchors(href.$phxW3AnchorTagName) \
		[string trim [$phxW3AnchorBox.nameframe.addr get]]
	    set tkW3HtAnchors(name.$phxW3AnchorTagName) \
		[string trim [$phxW3AnchorBox.nameframe.name get]]

	    if {$class == "para"} {
		tkW3HtMakeAnchor $phxW3AnchorTagName \
		    $tkW3HtAnchors(name.$phxW3AnchorTagName) \
		    $tkW3HtAnchors(href.$phxW3AnchorTagName)
	    } elseif {$class == "image"} {
		phxW3AnchorToggleImage $w 1 $phxW3AnchorTagName 0
		set class [phxW3WidgetClass $w]
	    }
	} else {
	    # We're changing the anchor
	    set tkW3HtAnchors(href.$phxW3AnchorTagName) \
		[string trim [$phxW3AnchorBox.nameframe.addr get]]
	    set tkW3HtAnchors(name.$phxW3AnchorTagName) \
		[string trim [$phxW3AnchorBox.nameframe.name get]]
	    tkW3HtMakeAnchor $phxW3AnchorTagName \
		$tkW3HtAnchors(name.$phxW3AnchorTagName) \
		$tkW3HtAnchors(href.$phxW3AnchorTagName)
	}

	# Second, configure the widget to refer to the anchor.
	if {$class == "para"} {
	    $w tag add Anchor \
		[lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]
	    $w tag add h$phxW3AnchorTagName \
		[lindex $phxW3AnchorText 0] [lindex $phxW3AnchorText 1]
	} elseif {$class == "image"} {
	    $w configure -command \
		"phxW3ModeSingleClickAnchor %W $phxW3AnchorTagName"
	    bind $w <Double-Button-1> \
		"phxW3ModeDoubleClickAnchor %W $phxW3AnchorTagName"
	}
    } else {
	# We're deleting an anchor.
	set tkW3HtAnchors(href.$phxW3AnchorTagName) ""
	set tkW3HtAnchors(name.$phxW3AnchorTagName) ""

	# For Text widgets: We deleted all tags above.
	# For Buttons: we must turn them into Labels.
	if {$class == "image"} {
	    phxW3AnchorToggleImage $w 0 0 0
	    set class [phxW3WidgetClass $w]
	}
    }
    phxW3SaveDirtyBit
}

# phxW3AnchorToggleImage
#
# w = Label/Button to be made into a Button/Label
# anchor = whether its being converted to an anchor
# index = index of anchor (if it is one)
# ismap = whether it is an ismap

proc phxW3AnchorToggleImage {w anchor index ismap} {
    global tkW3HtImages tkW3HtAnchors

    # Put in a dummy frame as a place holder.
    pack [frame .atiTemp] -after $w -side left

    # Extract the bitmap.  Get the index for information in tkW3HtImages.
    set bitmap \
	[string range [lindex [$w configure -bitmap] 4] 1 end]
    set i $tkW3HtImages($w)

    # Wipe out the old label/button.  Re-install the tkW3HtImages information.
    # We unset and reset tkW3HtImages($w) so that the destroy command (via
    # the binding) does not destroy the information in tkW3HtImages.
    unset tkW3HtImages($w)
    destroy $w
    set tkW3HtImages($w) $i

    # Create a new button/label according to specification
    phxW3WidgetCreateImage $w $bitmap $anchor $index $ismap
    pack $w -before .atiTemp -side left -anchor w
    if {$anchor} {
	set tkW3HtImages($i.anchor) $tkW3HtAnchors(href.$index)
	set tkW3HtImages($i.anchortag) $index
    } else {
	set tkW3HtImages($i.anchor) ""
	set tkW3HtImages($i.anchortag) ""
    }


    # Destroy the place holder
    destroy .atiTemp
}

proc phxW3AnchorCurrentHref {} {

    global phxW3AnchorBox

    return [string trim [$phxW3AnchorBox.nameframe.addr get]]
}

proc phxW3AnchorBroadcastHref {h} {
	#set h [phxW3InfoConvertToHTTP $h]
	phxW3AnchorChangeHref $h
	phxW3NavigateSendToTkWWW [list phxW3AnchorChangeHref $h]
}		

proc phxW3AnchorChangeHref {h} {

    global phxW3AnchorBox

    if {[info exists phxW3AnchorBox] && \
	[winfo exists $phxW3AnchorBox.nameframe.addr]} \
    {
	$phxW3AnchorBox.nameframe.addr delete 0 end
	$phxW3AnchorBox.nameframe.addr insert 0 $h
    }
}
